__author__ = 'luke'

from boto.kinesis.exceptions import ResourceInUseException, ResourceNotFoundException
import time


class Stream(object):

    active = 'ACTIVE'

    def __init__(self, conn, stream_name='dervish', shard_count=1, timeout=-1, retry_pause=1):
        self.conn = conn
        self.stream_name = stream_name
        self.timeout = timeout
        self.retry_pause = retry_pause
        print("finding stream %s" % stream_name)
        try:
            if self.status() == Stream.active:
                return
            else:
                self.wait_till_active(self.timeout, self.retry_pause)
        except ResourceInUseException as e:
            pass
        except ResourceNotFoundException as e:
            print("creating stream with %d shards" % (shard_count))
            conn.create_stream(stream_name=stream_name, shard_count=shard_count)
            self.wait_till_active(self.timeout, self.retry_pause)

    def put(self, key, data):
        if self.ready():
            start = time.time() * 1000
            self.conn.put_record(stream_name=self.stream_name, data=data, partition_key=key)
            dur = (time.time() * 1000) - start
            print("put time %d ms" % (dur))
        else:
            raise StreamNotReadyException("%s:%s" % (self.stream_name, self.status()))

    def status(self):
        r = self.conn.describe_stream(self.stream_name)
        description = r.get('StreamDescription')
        return description.get('StreamStatus')

    def wait_till_active(self, timeout=-1, retry_pause=1):
        print("waiting",end='')
        start = time.time()
        while not self.ready():
            if timeout >= 0 and time.time() > start + timeout:
                raise StreamTimeoutException
            time.sleep(retry_pause)
            print(".", end='')
        print("OK")
        print("stream is active")

    def ready(self):
        return self.status() == Stream.active


class StreamNotReadyException(Exception):
    pass


class StreamTimeoutException(Exception):
    pass



__author__ = 'lsamaha'

import random

class Words(object):

    def __init__(self, path='samples'):
        self.path = path
        self.nouns = self.load_words('nouns')
        self.verbs = self.load_words('verbs')
        self.adjectives = self.load_words('adjectives')
        self.adverbs = self.load_words('adverbs')

    def any_noun(self):
        return self.any(self.nouns)

    def any_verb(self):
        return self.any(self.verbs)

    def any_adjective(self):
        return self.any(self.adjectives)

    def any_adverb(self):
        return self.any(self.adverbs)

    def any(self, words):
        return random.choice(words).strip()

    def load_words(self, type):
        try:
            fname = "%s/%s" % (self.path, type)
            with open(fname) as f:
                words = f.readlines()
        except:
            words = ['fee','fie','foe','fum']
        return words



__author__ = 'lsamaha'

from dervisher.event import Event


class Dervisher(object):

    def __init__(self, product, post):
        self.product = product
        self.post = post

    def start(self, rpm):
        self.post.event(
            Event(event_class='start', event_type='service', subtype='whirl', env='dev', product=self.product))

    def stop(self):
        self.post.event(
            Event(event_class='stop', event_type='service', subtype='whirl', env='dev', product=self.product))




__author__ = 'lsamaha'

from dervisher.shard import Shard


class Post(object):

    def __init__(self, stream, shard=Shard()):
        self.stream = stream
        self.shard = shard

    def event(self, event):
        print("posting %s" % (event.tojson()))
        self.stream.put(self.shard.get(event), event.tojson())







__author__ = 'lsamaha'

import json
import time

class Event(object):

    def __init__(
            self, event_class, event_type, subtype, product, env, uow_uid=None, event_version = '0.1', pretty=False):
        self.event_class = event_class
        self.event_type = event_type
        self.subtype = subtype
        self.product = product
        self.env = env
        self.uow_uid = uow_uid
        self.pretty = pretty
        self.at = int(time.time() * 1000)
        self.event_version = event_version

    def tojson(self):
        return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, indent=4 if self.pretty else None)











#!/bin/bash
__author__ = 'lsamaha'

import sys, signal, time, random, uuid
from threading import Timer
from dervisher.dervisher import Dervisher
from dervisher.words import Words
from dervisher.event import Event
from dervisher.post import Post
from dervisher.stream import Stream
from boto.kinesis.layer1 import KinesisConnection

class Whirling(Dervisher):

    dance = ['back','forth']
    max_events_per_uow = 10

    def __init__(self):
        self.whirling = False
        self.timer = None
        self.count = 0
        self.words = Words()

    def start(self, rpm):
        Dervisher.start(self, rpm)
        self.timer = WhirlTimer(rpm, self.whirl)
        self.timer.start()
        self.whirling = True

    def stop(self):
        self.timer.stop()
        Dervisher.stop(self)

    def whirl(self):
        step = Whirling.dance[self.count % len(Whirling.dance)]
        uow_uid = str(uuid.uuid4())
        self.post.event(Event(event_class='start', event_type='whirl',
                              subtype=step, env='dev', product=self.product, uow_uid=uow_uid))
        for _ in range(random.randrange(0, Whirling.max_events_per_uow)):
            self.post.event(Event(event_class=self.words.any_verb(), event_type=self.words.any_noun(),
                              subtype=self.words.any_adverb(), env='dev', product=self.product, uow_uid=uow_uid))
        self.post.event(Event(event_class='complete', event_type='whirl',
                              subtype=step, env='dev', product=self.product, uow_uid=uow_uid))
        self.count += 1

    def sig(self, signal, frame):
        print("received signal %s" % signal)
        self.stop()

class WhirlTimer():

    def __init__(self, rpm, callback):
        self.timer = None
        self.wait_time = 60 / rpm
        self.callback = callback
        self.stopped = True
        self.start()

    def run(self):
        self.stopped = True
        self.start()
        self.callback()

    def start(self):
        if self.stopped:
            self.timer = Timer(self.wait_time, self.run)
            self.timer.start()
            self.stopped = False

    def stop(self):
        self.timer.cancel()
        self.stopped = True

def main():
    if len(sys.argv) < 2:
        print("usage: whirl rpm shards")
    else:
        rpm = int(sys.argv[1])
        shards = int(sys.argv[2])
        print("a dervisher starts whirling at %d rpm" % rpm)
        dervisher = Whirling(product = 'mimi', post=Post(stream=Stream(KinesisConnection(), shard_count=shards)))
        dervisher.start(rpm)
        signal.signal(signal.SIGINT, dervisher.sig)
        while(dervisher.whirling):
            time.sleep(1)

if __name__ == "__main__":
    main()


import os
from uuid import uuid4
from setuptools import setup
from pip.req import parse_requirements

install_reqs = parse_requirements('requirements.txt', session=uuid4())
reqs = [str(ir.req) for ir in install_reqs]

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "dervisher",
    version = "0.0.10",
    author = "Luke Samaha",
    author_email = "lwsamaha@gmail.com",
    description = ("Demonstrates use of AWS kinesis and dynamodb for application events."),
    license = "BSD",
    keywords = "aws kinesis dynamodb event",
    url = "http://packages.python.org/dervisher",
    packages = ['dervisher'],
    install_requires = reqs,
    entry_points = {'console_scripts':['dervisher = dervisher.whirling:main']}
)

classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: BSD License',
    'Programming Language :: Python :: 2.7'
]

